﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Trolleybus.DrawingObjects;
using Trolleybus.Entities;

namespace Trolleybus
{
    public partial class FormBusConfig : Form
    {
        /// <summary>
        /// Переменная-выбранный автобус
        /// </summary>
        DrawingBus? _bus = null;
        /// <summary>
        /// Событие
        /// </summary>
        private event Action<DrawingBus>? EventAddBus;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBusConfig()
        {
            InitializeComponent();
            panelRed.MouseDown += PanelColor_MouseDown;
            panelOrange.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelLightBlue.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            buttonCancel.Click += (s, e) => Close();
        }

        /// <summary>
        /// Отрисовка объекта
        /// </summary>
        private void DrawBus()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _bus?.SetPosition(5, 5);
            _bus?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawingBus> ev)
        {
            if (EventAddBus == null)
            {
                EventAddBus = ev;
            }
            else
            {
                EventAddBus += ev;
            }
        }

        /// <summary>
        /// Передача информации (имени Label) при нажатии на Label (простой / продвинутый объект)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name,
            DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому: текст) для панели
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации (имени Label)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _bus = new DrawingBus(
                   (int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value,
                   Color.White,
                   pictureBoxObject.Width, pictureBoxObject.Height);
                    break;

                case "labelAdvancedObject":
                    _bus = new DrawingTrolleybus(
                   (int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value,
                   Color.White, Color.Black,
                   checkBoxHorns.Checked, checkBoxBatteries.Checked,
                   pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawBus();
        }
        /// <summary>
        /// Передача цвета при нажатии на одну из Panel с цветом
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor,
            DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому: цвет) для Label с основным цветом
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelMainColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)) && _bus != null)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемого цвета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelMainColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = (Color)e.Data.GetData(typeof(Color));
            _bus.EntityBus.ChangeBodyColor(color);
            DrawBus();
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому: цвет) для Label с доп. цветом
        /// и проверка, что создаётся троллейбус
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelAdditionalColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)) && _bus != null && _bus is DrawingTrolleybus)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void LabelAdditionalColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = (Color)e.Data.GetData(typeof(Color));
            //Приведение к EntityTrolleybus для замены доп. цвета
            EntityTrolleybus? _trolleybus = _bus.EntityBus as EntityTrolleybus;
            _trolleybus.ChangeAdditionalColor(color);
            DrawBus();
        }

        /// <summary>
        /// Добавление автобуса
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            EventAddBus?.Invoke(_bus);
            Close();
        }
    }
}
